/* -------------------------------------------------------------------------------------------------------------------------
* Project           :C-to-FPGA Solution for Acclerating Tomographic Reconstruction
* File Name         :back_project_sw.c 
* Version           :1.0 (V1)
* (V1)Author        :Nikhil Subramanian
* (V1)Created       :Wed, Jan 28 2009
* References        :
* Description       :SW file. Includes code to measure execution time
*
******This file may not be distributed or modified without this header*****
*-------------------------------------------------------------------------------------------------------------------------*/


#define MONITOR
#include "co.h"
#include "mem_param.h"
#include <stdio.h>
#include <math.h>
#include <time.h>

// If compiling on the XD1000, uncomment the line below for timing information
//#define XD1000
#define DISP_MSG
//#define Time_Break

extern int image_mask[512][512];
/*Global variable defined in mask.c*/

extern float **sinogram_to_send;
/*Global variable defined in uwer_fbp.c*/

float image_hw[IMG_SIZE][IMG_SIZE]; 

//Declared Globally due to space restrictions on process
co_int16 sino_sending[SINO_SIZE][SINO_SIZE]; 
co_int16 image_fp[512][512];       // Image returned by FPGA. Supposed to be u10.6, FPGA returns 1s7.8
co_int32 image_acc[512][512];      //FPGA returns numbers with 8 fraction bits

#ifdef XD1000
#define TIME_MINUS(a,b) (((a).tv_sec-(b).tv_sec)+(double)((a).tv_nsec-(b).tv_nsec)/1000000000.0)
#endif  

  #ifdef XD1000
  struct timespec t1,t2,t3,t4,t5,t6,t7;
  float time_measured_cpu_to_sram;
  float time_measured_fpga_sram_inter;
  float time_measured_sram_to_cpu;
  float time_measured_cpu_accum;
  float total_compute_time;
  float post_proc_time;
  float total_time;
  #endif  



void Sw_Proc(co_memory datamem, co_signal start_comp, co_signal sw_read)
{   
    
    
    co_int32 sw_read_sig;
    int i,j,k,s,break_count =0;
    float max=0, min=0, absmax;

    for (k=0;k<SINO_SIZE;k++)
     for (s=0;s<SINO_SIZE;s++)
         {
          if(sinogram_to_send[k][s]>max)
            max = sinogram_to_send[k][s];  
          else if(sinogram_to_send[k][s]<min)
            min = sinogram_to_send[k][s];  
         }


    if(max>(-min))
     absmax=max;
    else
     absmax=-min;
 
    //Assign Sinogram in fixed point 1s5.10 format
    for (k=0;k<SINO_SIZE;k++)
     for (s=0;s<SINO_SIZE;s++)
       sino_sending[k][s] = (sinogram_to_send[k][s]/absmax)*pow(2,10);  //1s5.10    

    //Initialize Image to 0
    for (i=0;i<IMG_SIZE;i++)
     for (j=0;j<IMG_SIZE;j++)
        image_acc[i][j]=0;  

    //Time instant before start of data transfer from CPU to SRAM
    #ifdef XD1000
    clock_gettime(CLOCK_REALTIME, &t1);
    #endif

    #ifdef DISP_MSG
    printf("\n\nSW_PROC!!! Sending Sino Data to SRAM...");
    #endif

    co_memory_writeblock(datamem, 0, &sino_sending,SINO_SIZE*SINO_SIZE*sizeof(co_int16));         

    #ifdef DISP_MSG
    printf("\nSW_PROC!!! Done sending Sino Data to SRAM... FPGA Computations begin");
    #endif

    //Time instant at the end of data transfer to SRAM
    #ifdef Time_Break
    clock_gettime(CLOCK_REALTIME, &t2);
    time_measured_cpu_to_sram     = (float)TIME_MINUS(t2,t1);
    printf("\n\nTime for data transfer from CPU to SRAM : %fs\n", time_measured_cpu_to_sram);
    #endif

   
   while(1)
   {
    //Time instant before start of FPGA Computation 
    #ifdef XD1000
    clock_gettime(CLOCK_REALTIME, &t3);
    #endif

	co_signal_post(start_comp,0);

    #ifdef DISP_MSG
    printf("\nSW_PROC!!! Sent Start_comp Signal"); 
    #endif

    co_signal_wait(sw_read, &sw_read_sig);

    #ifdef DISP_MSG
    printf("\n\nSW_PROC!!! Received Proc engine Done from Proc Engine");
    #endif

    //Time instant at the end of FPGA computation
    #ifdef Time_Break
    clock_gettime(CLOCK_REALTIME, &t4);
    time_measured_sram_to_cpu = (float)TIME_MINUS(t4,t3);
    printf("\nTime for Computation         : %fs\n", time_measured_sram_to_cpu);
    #endif

	//co_memory_readblock(datamem,IMG_OFFSET, &image_fp,IMG_SIZE*IMG_SIZE*sizeof(co_int16)); 
    ++break_count;

    for (i=0;i<IMG_SIZE;i++)
        for (j=0;j<IMG_SIZE;j++)
            image_acc[i][j]+=image_fp[i][j];

    #ifdef DISP_MSG
    printf("\nSW_PROC!!!Accumulated Partial Image %d", break_count);
    #endif

    //Time instant at end of CPU Accumulation
    #ifdef Time_Break
    clock_gettime(CLOCK_REALTIME, &t5);
	time_measured_cpu_accum     = (float)TIME_MINUS(t5,t4);    
    printf("\nTime for CPU Accumulation: %fs\n", time_measured_sram_to_cpu);
    #endif 
    
    if(break_count==8)
    break;

  }
	co_memory_readblock(datamem,IMG_OFFSET, &image_fp,IMG_SIZE*IMG_SIZE*sizeof(co_int16)); 
    

    for (i=0;i<IMG_SIZE;i++)
        for (j=0;j<IMG_SIZE;j++)
            image_acc[i][j]+=image_fp[i][j];

    #ifdef XD1000
    clock_gettime(CLOCK_REALTIME, &t6);
	total_compute_time   = (float)TIME_MINUS(t6,t1);    
    printf("\nTotal compute time: %fs\n", total_compute_time);
    #endif 



    for (i=0;i<IMG_SIZE;i++)
        for (j=0;j<IMG_SIZE;j++)
            {   
              if(image_mask[i][j]==1)
              {
                if(FX2REAL32(image_acc[i][j],8)<0)
                 image_hw[i][j]=0;
                 else 
                 image_hw[i][j]=FX2REAL32(image_acc[i][j],8);
              }
              else
                image_hw[i][j]=0;
            }

    #ifdef XD1000
    clock_gettime(CLOCK_REALTIME, &t7);
	post_proc_time   = (float)TIME_MINUS(t7,t6);    
    total_time       = (float)TIME_MINUS(t7,t1);
    printf("\nTime for post proc: %fs\n", post_proc_time);
    printf("\nTotal Time: %fs\n", total_time);
    #endif 

}
