/* -------------------------------------------------------------------------------------------------------------------------
* Project           :C-to-FPGA Solution for Acclerating Tomographic Reconstruction
* File Name         :log.c (from project UW Emission Reconstruction) 
*                    Copyright (c) 2004 University of Washington - Imaging Research Laboratory. All rights reserved.
* Version           :1.0 (V1)
* (V1)Author        :Adam Alessio
* (V1)Created       :Wed, Jan 14 2004
* References        :
* Description       :Generates a log file
*
******This file may not be distributed or modified without this header*****
*-------------------------------------------------------------------------------------------------------------------------*/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <stdarg.h>

#include "log.h"

extern FILE *LogFile;

/*****************************************************************************
[NAME]
GetDateTime

[SYNOPSIS]
 void GetDateTime(char *str,
                  int DateTimeFormat);
 
[DESCRIPTION]

This function gets the current time and date, and returns it in the
pointer {\tt str}. The return string are formatted accordingly to {\tt
DateTime} Format. Three formats are avaliable as of now, {\tt
\_LongDate}, {\tt \_Time} and {\tt RealTime}. {\tt \_Longdate} returns the 
date and time in a string as `Day, dd. month, hh:mm:ss', {\tt \_Time} will
return `hh:mm:ss' and {\tt  \_RealTime} returns the number of seconds
from last midnight.

[USAGE]
{\tt GetDateTime(str,\_Time);}

Returns the current time in {\tt str} formatted as ``12:34:56''.

[REVISION]
Oct. 94, JJJ
1/20/03 AMA
*****************************************************************************/
void GetDateTime(char *str, int DateTimeFormat)
{
  time_t *timep;
  struct tm *times;

  if (!(timep=(time_t *)malloc(sizeof(time_t))))
    Error(" Memory allocation error (GetDateTime)");
  if (!(times=(struct tm *)malloc(sizeof(struct tm))))
    Error(" Memory allocation error (GetDateTime)");
  time(timep);
  times=localtime(timep);
  if (DateTimeFormat==_LongDate) 
    strftime(str, 55, "%A, %d. %B, %H.%M:%S", times);
  if (DateTimeFormat==_Date) 
    strftime(str, 8, "%y%m%d", times);
  if (DateTimeFormat==_Time) 
    strftime(str, 20,"%H.%M:%S", times);
  if (DateTimeFormat==_RealTime)
    sprintf(str,"%d",times->tm_sec+times->tm_min*60+times->tm_hour*3600);
}


/*****************************************************************************
[NAME]
OpenLog

[SYNOPSIS]
 void OpenLog(char* FileName);

[DESCRIPTION]

This function creates and opens a log file named {\tt
FileName.log}. Appends time and date to the top if the file.  If the
global variable {\tt DebugLevel} equals {\tt \_DHardCore} or {\tt
\_DNolog} no log will be opened. The file handle is the global
variable {\tt LogFile}.

[USAGE]
{\tt OpenLog("Test.log");}

Creates and opens the log file {\tt Test.log}.

[REVISION]
Oct. 94, JJJ
1/20/03 AMA
*****************************************************************************/
void OpenLog(char *FileName)
{
  char str[100];
  strcpy(LogFileName,FileName);
  strcat(LogFileName,".log");
  GetDateTime(str, _LongDate);
  if (!(LogFile=fopen(LogFileName, "w"))) 
  Error("Error opening LOG file");
  fprintf(LogFile,"--- START OF UWER LOG; %s ---\n", str);   
  fclose(LogFile);
  
}


/*****************************************************************************
[NAME]
CloseLog

[SYNOPSIS]
 void CloseLog()

[DESCRIPTION]

This function closes the log file and appends time and date to the end
of the file.

[USAGE]
{\tt CloseLog():}

Closes the current log file.

[REVISION]
Oct. 94, JJJ
1/20/03 AMA
*****************************************************************************/
void CloseLog()
{
  char str[100];

    if (!(LogFile=fopen(LogFileName, "a+"))) 
      Error("Error opening LOG file");
    GetDateTime(str, _Time);
    fprintf(LogFile,"---- Normal termination; %s ---\n", str);   
    fflush(LogFile);
    fclose(LogFile);
  
}


/*****************************************************************************
[NAME]
Print

[SYNOPSIS]
 void Print(int Level,
            char *fmt,
            ...);

[DESCRIPTION]

This function handles all the writing to the screen and to the
log. {\tt fmt} and {\tt ...} are the same arguments as to {\tt
print}. The variable {\tt \_Level} combined with the global variable
{\tt DebugLevel} chooses what should be written. {\tt DebugLevel}
takes precedence over {\tt Level}. The following choises exist

\begin{tabbing}
{\tt \_DHardCore} \== Nothing written.\\
{\tt \_DSilent}   \>= Nothing written on screen, full log.\\
{\tt \_DNolog}    \>= Full information on screen, no log.\\
{\tt \_DNormal}   \>= Some information on screen, full log.\\
{\tt \_DDebug}    \>= Full information both on screen and in log.
\end{tabbing}

[USAGE]
\verb+Print(_DNormal,"File not found '%%s' \n", TestFileName);+ 

Prints the above message both on the screen and in the log file.

[REVISION]
Oct. 94, JJJ
1/20/03 AMA
*****************************************************************************/
void Print(char *fmt, ...)
{ 
  char LogString[255];
  char str[32];
  va_list ap;

  va_start(ap,fmt);
  vsnprintf(LogString, sizeof(LogString),fmt, ap);  
  GetDateTime(str, _Time);
  
  if (!(LogFile=fopen(LogFileName, "a+"))) 
    Error("Error opening LOG file");
  
  fprintf(LogFile,"%s: %s",str,LogString);
  fclose(LogFile);
  
  vprintf(fmt,ap);
  fflush(stdout);
  fflush(LogFile);
  
  va_end(ap);
} 


/*****************************************************************************
[NAME]
Error

[SYNOPSIS]
 void Error(char *fmt,
            ...);

[DESCRIPTION]

This function is called when a fatal error occurs. It prints the error
message on the screen, closes the log and exits. {\tt fmt} and {\tt
...} are the same arguments as to {\tt print}.

[USAGE]
{\tt Error("Memory allocation problems");}

Prints the above message, closes the log and exits.


[REVISION]
Oct. 94, JJJ and PAP
1/20/03 AMA
*****************************************************************************/
void Error(char *fmt, ...)
{
  char LogString[255];
  char str[100];
  va_list ap;

  va_start(ap,fmt);
  vsprintf(LogString, fmt, ap); 
  va_end(ap);


  GetDateTime(str, _Time);
  if (!(LogFile=fopen(LogFileName, "a+"))) 
  printf("Error opening LOG file");
  fprintf(LogFile,"%s: %s\n",str,LogString);
  fprintf(LogFile,"---- UWER terminated with an error; %s ---\n", str);   
  fclose(LogFile);

  printf(LogString);
  printf("\n");
  exit(1);
}

