/* -------------------------------------------------------------------------------------------------------------------------
* Project           :C-to-FPGA Solution for Acclerating Tomographic Reconstruction
* File Name         :uwer_ecat_io.c (from project UW Emission Reconstruction) 
*                    Copyright (c) 2004 University of Washington - Imaging Research Laboratory. All rights reserved.
* Version           :1.0 (V1)
* (V1)Author        :Adam Alessio
* (V1)Created       :Wed, Jan 14 2004
* References        :
* Description       :Functions for reading ecat data
*
******This file may not be distributed or modified without this header*****
*-------------------------------------------------------------------------------------------------------------------------*/

#include "uwer_ecat_io.h"
short read2int(FILE *fp, int record, int offset);
int read4int(FILE *fp, int record, int offset);
float read4real(FILE *fp, int record, int offset);

void read_input_ecat_sinogram(struct sino_info *sinoInfo,FILE *fp, struct exec_parameters *params)    
{
  int cnts,i;
  short file_type, num_planes;
  int startRec,endRec;
  float plane_sep;
  long dir[128];
  short nAngles, nProjs, num_dimensions,nz,data_type, storageOrder;
  float tres, ares,scale;
  int frame, plane, angle,proj,index, prompts, delays;
  float *tempFloat;
  int little_endian, verbose = 1;
  
  fseek(fp, 50,SEEK_SET);
  cnts = fread(&file_type,sizeof(short),1,fp);
  if(verbose) printf("file_type = %d\n", file_type);
 
  
  fseek(fp, 512,SEEK_SET);
  cnts = fread(dir,sizeof(long),128,fp);
  for(i=0;i<32;i++){
    if(verbose) printf("dir[%d] = ",i);
    if(dir[i*4+3] == 1) // matrix exists
      {
	startRec = dir[i*4+1];
	endRec = dir[i*4+2];
	//break;
      }
    for(cnts = 0;cnts<4;cnts++)
      if(verbose) printf("%ld   ",dir[i*4+cnts]);
    if(verbose) printf("\n");
  }
  if(verbose) printf("start,end = %d, %d\n", startRec, endRec);
  //  fseek(fp, 512*2+2,SEEK_SET);
  // cnts = fread(&num_dimensions,sizeof(short),1,fp);
  data_type = read2int(fp,startRec,0);
  if(verbose) printf("data_type = %d\n",data_type);

  if(data_type<=4)      // This is little_endian data
    little_endian = 1;
  else
    little_endian = 0;
  if(verbose) printf("data_type = %d\n",data_type);
  
  num_dimensions = read2int(fp,startRec,2);
  
  if(verbose) printf("num_dimensions=%d\n",num_dimensions);

  //fseek(fp, 512*2+4,SEEK_SET);
  //cnts = fread(&nProjs,sizeof(short),1,fp);

  nProjs = read2int(fp,startRec,4);
  if(verbose) printf("num_projs=%d\n",nProjs);

  nAngles=read2int(fp, startRec,6);
  if(verbose) printf("num_angles=%d\n",nAngles);

  nz = read2int(fp,startRec,10);
  if(verbose) printf("nz = %d\n",nz);

  tres = read4real(fp,startRec,144)/100.0;   // Convert from cm to m
  ares = read4real(fp,startRec,152)/100.0;
  if(verbose) printf("tres = %f, ares = %f\n",tres,ares);

  scale = read4real(fp,startRec,184);
  if(verbose) printf("scale = %f\n",scale);
  storageOrder = read2int(fp,startRec,140);
  if(verbose) printf("storageOrder = %d\n",storageOrder);

  delays = read4int(fp,startRec,196);
  prompts = read4int(fp,startRec,192);
  if(verbose) printf("Total prompts = %d, delays = %d\n",prompts,delays);
  sinoInfo->nPlanes = (int)nz;
  sinoInfo->nAngles = (int)nAngles;
  sinoInfo->nProjs = (int)nProjs;
  params->tfov = (double)tres*(double)(nProjs-1);
  sinoInfo->nFrames = 1;  // Could pull from matrix file  

  tempFloat = (float*)get_spc(sinoInfo->nPlanes*sinoInfo->nAngles*sinoInfo->nProjs,sizeof(float));

  fseek(fp,512*(startRec+1),SEEK_SET);
  cnts = fread(tempFloat, sizeof(float), sinoInfo->nPlanes*sinoInfo->nAngles*sinoInfo->nProjs,fp);

  i = 1;
  if((little_endian) && (!(*(char*)&i))){
    Print("Big endian machine, reading little endian data-> swap\n");
    for(index = 0;index<cnts;index++)
      SWAP_FLOAT(tempFloat[index]);
  }
  if(!(little_endian) && ((*(char*)&i))){
    Print("Little endian machine, reading big endian data->swap\n");
    for(index = 0;index<cnts;index++)
      SWAP_FLOAT(tempFloat[index]);
  }  
  
  
  sinoInfo->sinogramData = (float****)multialloc(sizeof(float),4,sinoInfo->nFrames,sinoInfo->nPlanes,sinoInfo->nAngles,sinoInfo->nProjs);
  index = 0;
  for(frame=0;frame<sinoInfo->nFrames;frame++) 
    for (angle=0;angle<sinoInfo->nAngles;angle++)
      for (plane=0;plane<sinoInfo->nPlanes;plane++)
	for (proj = 0; proj<sinoInfo->nProjs;proj++)
	  sinoInfo->sinogramData[frame][plane][angle][proj] =scale*(float)tempFloat[index++];

  
  free((void*)tempFloat);

  
}


short read2int(FILE *fp, int record, int offset)
{
  short val;
  fseek(fp,512*(record-1)+offset,SEEK_SET);
  fread(&val,sizeof(short),1,fp);
  return(val);
}

int read4int(FILE *fp, int record, int offset)
{
  int val;
  fseek(fp,512*(record-1)+offset,SEEK_SET);
  fread(&val,sizeof(int),1,fp);
  return(val);
}


float read4real(FILE *fp, int record, int offset)
{
  float val;
  fseek(fp,512*(record-1)+offset,SEEK_SET);
  fread(&val,sizeof(float),1,fp);
  return(val);
}

