/* -------------------------------------------------------------------------------------------------------------------------
* Project           :C-to-FPGA Solution for Acclerating Tomographic Reconstruction
* File Name         :uwer_main.c from project (UW Emission Reconstruction) 
*                    Copyright (c) 2004 University of Washington - Imaging Research Laboratory. All rights reserved.
* Version           :2.0 (V2)
* (V1)Author        :Adam Alessio
* (V1)Created       :Wed, Jan 14 2004
* (V2)Author        :Nikhil Subramanian
* (V3)Created       :Wed, Jan 14 2009
* References        :
* Description       :This file is the entry point for the UW Emission Reconstruction codebase.
*                    V2 removes functionality to perform iterative reconstruction. This code performs forward projection 
*                    and backprojection
* Notes             :to perform backprojection use the following command line arguments
*                    -t 3 -r 512 -R 0 -c 1.0 -S  forward1024.fld -O recon.fld
*
******This file may not be distributed or modified without this header*****
*-------------------------------------------------------------------------------------------------------------------------*/



#include <stdio.h>
#include <math.h>
#include <string.h>

#include "uwer_main.h"

int main(int argc, char **argv)
{
	struct exec_parameters params;  
	struct sino_info sinoInfo;
	struct system_matrix_info systemInfo;
	struct recon_image_info imageInfo;
	
	parse_cmdline_configfile(&params,argc,argv);   //  parse settings for execution and assign to structure exec_params

	/*-------------------------------------------*/
	/*----- Perform Reconstruction Operation----*/
	if(params.reconType<=4){
	
	read_input_sinogram(&params, &sinoInfo);
		
	prepare_recon_image(&params,&sinoInfo,&imageInfo);
	
	/*------- Choose Type of Recon --------*/
	
	if(params.reconType==3)
	  /*  Perform FBP */
	  fbp_recon(&sinoInfo, &imageInfo,&params);
	else
	  usage_error();
	
	output_recon_image(&params,&imageInfo);

	}
	/*---------------------------------------------------------------------*/
	/*----- Perform Simple Forward Projection Operation-For Testing/Simulation----*/
	if(params.reconType==5){

	  read_input_avs_image(&imageInfo, &params);
	  prepare_forward_projections(&params,&sinoInfo,&imageInfo);
	  forwardSimulation(&sinoInfo,&imageInfo,&params);
	  
	}
	  
	
	return(0);
} // End main()
